@@ -28,13 +28,13 @@
 # SOFTWARE.
 # ==============================================================================
 
-from abc import abstractmethod
+from abc import ABC, abstractmethod
 from typing import Optional
 
 from .status import ConnectionStatus
 
 
-class Connection:
+class Connection(ABC):
 
     """
         Max length of memory cache
@@ -88,12 +88,24 @@ class Connection:
         """ Get remote address (host:port) """
         raise NotImplemented
 
-    @property
-    def alive(self) -> bool:
-        """ Check whether connection is alive: the thread is still running """
-        raise NotImplemented
-
     @property
     def status(self) -> ConnectionStatus:
         """ Get status """
         raise NotImplemented
+
+    #
+    #   Runnable
+    #
+
+    @property
+    def running(self) -> bool:
+        """ Check whether connection is still running """
+        raise NotImplemented
+
+    def run(self):
+        """ Run loop to be started in a thread """
+        pass
+
+    def stop(self):
+        """ Close the connection """
+        pass
